import { StateSpecNode } from "./graph";
export type InitFuncEnv = {
    $props: Record<string, any>;
    $state: Record<string, any>;
    $queries?: Record<string, any>;
    $ctx?: Record<string, any>;
    $refs?: Record<string, any>;
};
export type DollarStateEnv = Omit<InitFuncEnv, "$state">;
export type NoUndefinedField<T> = {
    [P in keyof T]-?: T[P];
};
export type InitFunc<T> = (env: NoUndefinedField<InitFuncEnv>) => T;
export type DeprecatedInitFunc<T> = ($props: Record<string, any>, $state: $State, $ctx: Record<string, any>) => T;
export type ObjectPath = (string | number)[];
export interface $StateSpec<T> {
    path: string;
    initFunc?: InitFunc<T>;
    initVal?: T;
    type: "private" | "readonly" | "writable";
    valueProp?: string;
    onChangeProp?: string;
    isImmutable?: boolean;
    variableType: "text" | "number" | "boolean" | "array" | "object" | "variant" | "dateString" | "dateRangeStrings";
    initFuncHash?: string;
    refName?: string;
    onMutate?: (stateValue: T, $ref: any) => void;
}
export interface $State {
    [key: string]: any;
    registerInitFunc?: (path: string, f: InitFunc<any>, repetitonIndex?: number[], overrideEnv?: DollarStateEnv) => any;
}
export declare const ARRAY_SYMBOL: unique symbol;
export declare const PLASMIC_STATE_PROXY_SYMBOL: unique symbol;
export declare const UNINITIALIZED: unique symbol;
export interface Internal$StateSpec<T> extends $StateSpec<T> {
    isRepeated: boolean;
    pathObj: (string | symbol)[];
}
export interface Internal$StateInstance {
    path: ObjectPath;
    specKey: string;
}
export interface StateCell<T> {
    initialValue?: T | symbol;
    node: StateSpecNode<any>;
    path: ObjectPath;
    initFunc?: InitFunc<T>;
    listeners: (() => void)[];
    initFuncHash: string;
    overrideEnv?: NoUndefinedField<DollarStateEnv>;
}
export interface Internal$State {
    registrationsQueue: {
        node: StateSpecNode<any>;
        path: ObjectPath;
        f: InitFunc<any>;
        overrideEnv?: NoUndefinedField<DollarStateEnv>;
    }[];
    stateValues: Record<string, any>;
    env: NoUndefinedField<DollarStateEnv>;
    rootSpecTree: StateSpecNode<any>;
    specTreeLeaves: StateSpecNode<any>[];
    specs: $StateSpec<any>[];
    stateInitializationEnv: {
        stack: string[];
        visited: Set<string>;
    };
    initializedLeafPaths: Set<string>;
}
