import * as React from "react";
import { AnyPlasmicClass, PlasmicClassArgs, PlasmicClassOverrides, PlasmicClassVariants, VariantDef } from "../plume-utils";
interface PlumeCommonProps {
    showStartIcon?: boolean;
    showEndIcon?: boolean;
    startIcon?: React.ReactNode;
    endIcon?: React.ReactNode;
    children?: React.ReactNode;
    isDisabled?: boolean;
}
interface HtmlButtonProps extends Omit<React.ComponentProps<"button">, "ref" | "disabled"> {
}
interface HtmlAnchorProps extends Omit<React.ComponentProps<"a">, "ref" | "href" | "target"> {
}
interface PlumeActualButtonProps {
    submitsForm?: boolean;
}
interface PlumeAnchorProps {
    link?: string;
    target?: React.ComponentProps<"a">["target"] | boolean;
}
export type PlumeButtonProps = PlumeCommonProps & PlumeActualButtonProps & PlumeAnchorProps;
export type BaseButtonProps = PlumeButtonProps & HtmlButtonProps & HtmlAnchorProps;
type AllButtonProps = PlumeCommonProps & PlumeActualButtonProps & HtmlButtonProps;
type AllAnchorProps = PlumeCommonProps & PlumeAnchorProps & HtmlAnchorProps;
export type HtmlAnchorOnlyProps = Exclude<keyof AllAnchorProps, keyof AllButtonProps>;
export type HtmlButtonOnlyProps = Exclude<keyof AllButtonProps, keyof AllAnchorProps>;
export type ButtonRef = React.Ref<HTMLButtonElement | HTMLAnchorElement>;
interface ButtonConfig<C extends AnyPlasmicClass> {
    showStartIconVariant: VariantDef<PlasmicClassVariants<C>>;
    showEndIconVariant?: VariantDef<PlasmicClassVariants<C>>;
    isDisabledVariant?: VariantDef<PlasmicClassVariants<C>>;
    startIconSlot?: keyof PlasmicClassArgs<C>;
    endIconSlot?: keyof PlasmicClassArgs<C>;
    contentSlot: keyof PlasmicClassArgs<C>;
    root: keyof PlasmicClassOverrides<C>;
}
export declare function useButton<P extends PlumeButtonProps, C extends AnyPlasmicClass>(plasmicClass: C, props: P, config: ButtonConfig<C>, ref?: ButtonRef): {
    plasmicProps: {
        variants: PlasmicClassVariants<C>;
        args: PlasmicClassArgs<C>;
        overrides: PlasmicClassOverrides<C>;
    };
};
export {};
